import json
import yaml
import requests
from typing import Any, List
import os
from .types import ChatgptAssistantMessage, ChatgptFunctionMessage, CallableFunction, OTPHash

class OpenPlugin:
    def __init__(self, plugin_name: str):
        print("CWD", os.getcwd())
        self.root_url: str = ''
        self.name: str = plugin_name
        self.description: str = ''
        self.openapi: Any = None
        self.manifest: Any = None
        self.functions: List[CallableFunction] = []
        self.operationid_to_path_hash: OTPHash = {}
        self.init(plugin_name)

    def init(self, plugin_name: str) -> None:
        base_dir = os.path.dirname(os.path.realpath(__file__))
        plugins_file_path = os.path.join(base_dir, "plugins.json")
        with open(plugins_file_path) as f:
            plugins = json.load(f)
        self.root_url = plugins[plugin_name]
        self.manifest = self.fetch_manifest(self.root_url)
        self.openapi = self.fetch_openapi(self.root_url)
        self.functions = self.openapi_to_functions(self.openapi)
        self.operationid_to_path_hash = self.openapi_to_otphash(self.openapi)

    def fetch_manifest(self, root_url: str) -> Any:
        response = requests.get(root_url + "/.well-known/ai-plugin.json")
        response.raise_for_status()  # Raise exception if the request failed
        json_response = response.json()
        self.name = json_response["name_for_model"]
        self.description = json_response["description_for_model"]
        return json_response

    def fetch_openapi(self, root_url: str) -> Any:
        response = requests.get(root_url + "/openapi.yaml")
        response.raise_for_status()  # Raise exception if the request failed
        return yaml.safe_load(response.text)  # Parse as YAML, regardless of whether it's JSON or YAML


    def openapi_to_functions(self, openapi: Any) -> List[CallableFunction]:
        # Needs implementation
        return []

    def openapi_to_otphash(self, openapi: Any) -> OTPHash:
        # Needs implementation
        return {}

    def is_plugin_function(self, assistant_response: ChatgptAssistantMessage) -> bool:
        pass

    def fetch_plugin_api(self, assistant_response: ChatgptAssistantMessage) -> ChatgptFunctionMessage:
        pass

    def aggregate_functions(self, callable_functions: List[CallableFunction]) -> CallableFunction:
        pass
